package fr.inria.diagen.libPrinterDisplay;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Collection;

/**
 * This class is a front-end to the little display provided by PJL printers
 * (like out HP 4200)
 */
public class PJLPrinterDisplay {

	private static final String HP_HEADER = "%-12345X";
	private static final int NB_LINES = 4;
	private static final int CHAR_PER_LINE = 20;

	private Socket socket;

	public PJLPrinterDisplay(Socket socket) {
		super();
		this.socket = socket;
	}

	public PJLPrinterDisplay(InetAddress address) throws IOException {
		this(new Socket(address, 9100));
	}

	private void addStringToByteArray(Collection<Byte> bytes, String string) {
		for (char character : string.toCharArray()) {
			bytes.add((byte) character);
		}
	}

	/**
	 * Puts the initial READY message back
	 *
	 * @throws IOException
	 */
	public void resetDisplay() throws IOException {
		privateDisplay("");
	}

	public void display(String... lines) throws IOException {
		StringBuilder builder = new StringBuilder();

		if (lines.length > NB_LINES) {
			throw new RuntimeException("Too many lines");
		}

		int nb_empty_lines = (NB_LINES - lines.length) / 2;

		for (int i = 0; i < nb_empty_lines; ++i) {
			builder.append(emptyLine());
		}

		for (String line : lines) {
			builder.append(center(line));
		}

		for (int i = 0; i < nb_empty_lines; ++i) {
			builder.append(emptyLine());
		}

		debugResult(builder.toString());
		privateDisplay(builder.toString());
	}

	/**
	 * Prints given string on System.out
	 */
	private void debugResult(String string) {
		int i = 0;
		for (char character : string.toCharArray()) {
			System.out.print(character);
			if ((++i % 20) == 0) {
				System.out.println("|");
			}
		}
	}

	private String emptyLine() {
		StringBuilder builder = new StringBuilder();

		for (int i = 0; i < CHAR_PER_LINE; i++) {
			builder.append(" ");
		}

		return builder.toString();
	}

	private String center(String line) {
		StringBuilder builder = new StringBuilder();

		if (line.length() > CHAR_PER_LINE) {
			throw new RuntimeException("Line too long");
		}

		int numberOfSpaces = (CHAR_PER_LINE - line.length()) / 2;

		for (int i = 0; i < numberOfSpaces; ++i) {
			builder.append(" ");
		}

		builder.append(line);

		for (int i = 0; i < numberOfSpaces; ++i) {
			builder.append(" ");
		}

		// Ensure the string is of the length CHAR_PER_LINE
		if ((line.length() % 2) != (CHAR_PER_LINE % 2)) {
			builder.append(" ");
		}

		return builder.toString();
	}

	/**
	 * Really sends data to the printer
	 *
	 * @param line
	 */
	private void privateDisplay(String line) {
		byte[] realBytes = pjlCommandFor(line);

		try {
			socket.getOutputStream().write(realBytes);
		} catch (IOException e) {
			throw new RuntimeException("Sending the message to the printer failed", e);
		}
	}

	/**
	 * Creates a PJL command (Printer Job Language) to display the given string
	 */
	private byte[] pjlCommandFor(String line) {

		checkStringValidity(line);

		ArrayList<Byte> bytes = new ArrayList<Byte>();
		bytes.add((byte) 27); // ESC
		addStringToByteArray(bytes, HP_HEADER);
		addStringToByteArray(bytes, "@PJL JOB");
		bytes.add((byte) 13); // CR
		bytes.add((byte) 10); // LF
		addStringToByteArray(bytes, "@PJL RDYMSG DISPLAY=");
		bytes.add((byte) 34); // "
		addStringToByteArray(bytes, line);
		bytes.add((byte) 34); // "
		bytes.add((byte) 13); // CR
		bytes.add((byte) 10); // LF
		addStringToByteArray(bytes, "@PJL EOJ");
		bytes.add((byte) 13); // CR
		bytes.add((byte) 10); // LF
		bytes.add((byte) 27); // ESC
		addStringToByteArray(bytes, HP_HEADER);

		byte[] realBytes = new byte[bytes.size()];
		int i = 0;
		for (Byte byte1 : bytes) {
			realBytes[i++] = byte1;
		}
		return realBytes;
	}

	private void checkStringValidity(String line) {
		if (line.length() > NB_LINES * CHAR_PER_LINE) {
			throw new RuntimeException("Too much characters. Max is " + (NB_LINES * CHAR_PER_LINE));
		}

		for (char character : line.toCharArray()) {
			checkCaracterValidity(character);
		}
	}

	/**
	 * Checks if character is a printable character or a space but not a quote
	 */
	private void checkCaracterValidity(char character) {
		// to do: implement checkCaracterValidity
	}

	public static void main(String[] args) throws IOException {
		PJLPrinterDisplay printerDisplay = new PJLPrinterDisplay(InetAddress.getByName("d2r2"));
		printerDisplay.display("I love", "DiaGen");
	}
}